@@ -47,7 +47,7 @@
 #'
 #' @importFrom graphics legend matplot polygon
 #' @importFrom grDevices gray.colors
-#' @importFrom stats is.ts time
+#' @importFrom stats is.ts terms time
 #'
 #' @examples
 #' areaplot(rpois(10,40))
@@ -82,6 +82,7 @@
 #' abline(v=1983+1/12, lty=3)
 #'
 #' # legend
+#' require(MASS)
 #' areaplot(table(Aids2$age, Aids2$sex), legend=TRUE)
 #' areaplot(WorldPhones, legend=TRUE, args.legend=list(x="topleft"))
 #'
@@ -186,8 +187,8 @@ areaplot.default <- function(x, y=NULL, prop=FALSE, add=FALSE, xlab=NULL,
     }
     else
     {
-      args.legend1 <- list(x="topright", legend=rev(legend), fill=rev(col), bty="n",
-                           inset=0.02)
+      args.legend1 <- list(x="topright", legend=rev(legend), fill=rev(col),
+                           bty="n", inset=0.02)
       args.legend1[names(args.legend)] <- args.legend
       do.call("legend", args.legend1)
     }
@@ -200,25 +201,27 @@ areaplot.default <- function(x, y=NULL, prop=FALSE, add=FALSE, xlab=NULL,
 #' @export
 #' @export areaplot.formula
 
-areaplot.formula <- function (formula, data, subset, na.action=NULL, ...)
+areaplot.formula <- function(formula, data, subset, na.action=NULL, ...)
 {
   m <- match.call(expand.dots=FALSE)
   if(is.matrix(eval(m$data,parent.frame())))
     m$data <- as.data.frame(data)
   m$... <- NULL
-  m[[1]] <- as.name("model.frame")
-  if(as.character(formula[[2]]=="."))
+  m[[1]] <- quote(model.frame)
+  if(formula[[2]] == ".")
   {
-    rhs <- unlist(strsplit(deparse(formula[[3]])," *[:+] *"))
-    lhs <- sprintf("cbind(%s)", paste(setdiff(names(data),rhs),collapse=","))
-    m[[2]][[2]] <- parse(text=lhs)[[1]]
+    ## LHS is .
+    rhs <- as.list(attr(terms(formula[-2]),"variables")[-1])
+    lhs <- as.call(c(quote(cbind), setdiff(lapply(names(data),as.name),rhs)))
+    formula[[2L]] <- lhs
+    m[[2L]] <- formula
   }
 
   mf <- eval(m, parent.frame())
   if(is.matrix(mf[[1]]))
   {
+    ## LHS is cbind()
     lhs <- as.data.frame(mf[[1]])
-    names(lhs) <- as.character(m[[2]][[2]])[-1]
     areaplot.default(cbind(mf[-1],lhs), ...)
   }
   else
